[ng-cloak]{display:none!important}.clickable,.sidebar li{cursor:pointer}pre{font-size:14px}.m-b-0{margin-bottom:0!important}.m-b-1{margin-bottom:1rem!important}.m-b-2{margin-bottom:2rem!important}.m-b-3{margin-bottom:3rem!important}.w-100{width:100%!important}.w-50{width:50%!important}.clear-right{clear:right}.tagline{margin-top:-20px;margin-bottom:20px}.breadcrumb{padding-top:3px;padding-bottom:3px}.nav-tabs{margin-bottom:20px}.ace_editor{font-size:16px!important;border:1px solid #ebebeb;border-radius:4px}legend{width:auto;margin:0;border:0}body,html{height:100%;background:inherit}#page-wrapper{min-height:100%}.icon-color{color:#000!important}#requestTable{margin-bottom:35px}.child-row{background-color:#f9f9f9}.child-link{margin-left:5px}#chevron{cursor:pointer;transition:all .25s linear}.fa-chevron-right{margin-right:8px}.chevron-rotate{transform:rotate(90deg)}#systems-container{display:flex;flex-flow:row wrap;justify-content:space-around;align-items:stretch;padding-bottom:20px}#systems-container .container-item{display:flex;flex-direction:column;justify-content:space-between;margin:10px;min-width:30%;max-width:30%}#systems-container .container-item h3{margin-top:10px;margin-bottom:10px}.system-table{padding-left:0;padding-right:0}.system-table.table-header{font-weight:700;text-align:right;padding-right:8px}#systems-container .container-item.empty{height:0;min-width:30%;max-width:30%}#formattedOutput.ng-hide,#rawOutput.ng-hide{opacity:0}#formattedOutput.ng-hide-add,#formattedOutput.ng-hide-remove,#rawOutput.ng-hide-add,#rawOutput.ng-hide-remove{transition:all 0s linear}#formattedOutput.ng-hide-add-active,#rawOutput.ng-hide-add-active{transition:all .1s linear}#formattedOutput.ng-hide-remove-active,#rawOutput.ng-hide-remove-active{transition:all .1s linear;transition-delay:.2s}#formattedOutput.ng-hide-animate,#rawOutput.ng-hide-animate{position:absolute;width:100%}.show-more{transition:max-height .3s;overflow:hidden}table.dataTable tbody td,table.table-bordered.dataTable tbody td{vertical-align:middle}table.dataTable{table-layout:fixed;word-wrap:break-word}#th_system{width:10%}#th_command{width:25%}#th_version{width:10%}#th_description{width:45%}#th_happen{width:10%}.btn,.word-wrap-button{white-space:normal!important;word-wrap:break-word}.dropdown-container{position:relative;cursor:default}.dropdown-menu{min-width:110px}.select.dropdown-menu{max-height:195px;overflow:auto}.red-border{border:2px solid red;border-radius:4px}.navbar-nav>.red-border{margin:-2px}#alertClose{padding:10px;position:absolute;top:0;right:0}#loading i{margin-top:10px}